/******************************************************************************
  (c) Copyright 2002-2008, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      
 *****************************************************************************/
package org.netkernel.urii;

import org.netkernel.container.IKernel;
import org.netkernel.request.IAsyncRequestee;

/** A space is a logical container in which resources can be located
 * and requested.
 * <p>
 * Physical code represented by an endpoint is responsible for the creation
 * of resource representation to satisfy logical level resource requests.
 * <p>
 * A space has life-cycle events that are executed in the following order:
 * <ol>
 * <li>onLinkspace()</li>
 * <li>onCommissionspace()</li>
 * <li>any number of requests to the IAsynchronousRequestee interface</li>
 * <li>preDecommisionspace()</li>
 * <li>onDecommissionspace()</li>
 * </ol>
 * At any time the isCommissioned() method may be called to determine if the space
 * is in a state to accept resource requests. Once a space has been decommissioned
 * it will not be re-commissioned.
 *
 * @author  tab
 */
public interface ISpace extends IAsyncRequestee
{
	/** Called to perform any inter-spatial linking in preparation for commissioning.
   * <p>
   * This method should be used to initialise the space classloader.
   *
   * No assumption should be made that other space can be used in any way (especially
   * using their classloader.
   * <p>
   * cross space linking prior to commissioning. This method
	 *  should also initialise it's classloader if it has one here. It should not assume that
	 *  it can access any other spaces yet though or even use their classloaders - that
	 *  should wait for commission.

	 * @param aKernel
	 * @throws Exception
	 */
	void onLinkSpace(IKernel aKernel) throws Exception;	
	
	/** Called to commission this space.
   * <p>
   * During the commissioning phase endpoint may be initialised.
   *
   * When this method returns the isCommissioned() method will return true
   * and the space is ready to accept resource requests.

   * @param aKernel
	 * @throws Exception any exception can be thrown and must be handled by caller
	 */
	void onCommissionSpace(IKernel aKernel) throws Exception;
	
	/** Called after a space is commissioned and before external requests are being
	 * processed. It is valid to issue subrequests from this call.
	 * @throws Exception
	 */
	void postCommissionSpace() throws Exception;
	
	/** Called before a space is decommission and before the kernel is paused. It is
	 * valid to issue subrequests from this call.
	 * @throws Exception
	 */
	void preDecommissionSpace() throws Exception;
	
	/** Called to decommission the space
	 * @throws Exception any exception can be thrown and must be handled by caller
	 */
	void onDecommissionSpace() throws Exception;
	
	/** Indicates if this space has been commissioned and is ready to receive requests.
   *
   * @return Commission status.
   */
	boolean isCommissioned();
	
	/** Return an optional classloader which should be used by code executing within this space.
	 * May return null if this space doesn't need it's own classloader and doesn't want to expose
	 * classes to importers.
	 */
	ClassLoaderWithExports getClassLoader();
	
	/** Should return true only for spaces that will return equivalent endpoints to all requests
	 * such that a request to either would return the same response. A safe default is the default
	 * object identity.
	 * @param aOther
	 * @return
	 */
	boolean equals(Object aOther);
	
	/** Should return true only for spaces that will return same endpoints to all requests even
	 * if those endpoints may return different value. I.e. all the same resource identifiers are exposed
	 * but they may refer to logically different resources due being in a different space.
	 * @param aOther
	 * @return
	 */
	boolean resolveEquals(Object aOther);
	
	/** Spaces which are equal should return the same hash code. It is particularly important
	 * to implement this for dynamic spaces which may be in scope of cachable resources.
	 * @return
	 */
	int hashCode();
	
}